@@ -62,7 +62,7 @@ class _PerspectiveDateValidator(object):
 
     def to_date_components(self, obj):
         '''Return a dictionary of string keys and integer values for `year`,
-        `month`, and `day`.
+        `month` (from 0 - 11), and `day`.
 
         This method converts both datetime.date and numpy.datetime64 objects
         that contain datetime.date.
@@ -85,9 +85,12 @@ class _PerspectiveDateValidator(object):
             if (six.PY2 and isinstance(obj, long)) or isinstance(obj, int):
                 obj = datetime.fromtimestamp(obj / 1000000000)
 
+        # Perspective stores month in `t_date` as an integer [0-11],
+        # while Python stores month as [1-12], so decrement the Python value
+        # before Perspective attempts to use it.
         return {
             "year": obj.year,
-            "month": obj.month,
+            "month": obj.month - 1,
             "day": obj.day
         }
 
@@ -150,16 +153,32 @@ class _PerspectiveDateValidator(object):
             if isinstance(obj, int):
                 return round(obj / 1000000)
 
-        if isinstance(obj, (int, float)):
+        if isinstance(obj, (int, float, numpy.integer, numpy.float)):
             return _normalize_timestamp(obj)
 
+        timetuple = getattr(obj, to_timetuple)()
+
+        is_datetime_min = timetuple.tm_year == 1 and timetuple.tm_mon == 1 \
+            and timetuple.tm_mday == 1 and timetuple.tm_hour == 0 \
+            and timetuple.tm_min == 0 and timetuple.tm_sec == 0
+
+        if is_datetime_min:
+            # Return beginning of epoch when datetime is datetime.min
+            return 0
+
+        if timetuple.tm_year < 1900:
+            # Use calendar.timegm to do conversion between timetuple and
+            # seconds timestamp
+            converter = timegm
+
         # At this point, aware datetime objects are in UTC, and naive datetime
         # objects have not been modified. When the timestamp is serialized
         # using `to_format`, it will be in *local time* - Pybind will
         # automatically localize any conversion to `datetime.datetime`
         # from C++ to Python.
-        return int((converter(
-            getattr(obj, to_timetuple)()) + obj.microsecond / 1000000.0) * 1000)
+        seconds_timestamp = (converter(timetuple) + obj.microsecond / 1000000.0)
+        ms_timestamp = int(seconds_timestamp * 1000)
+        return ms_timestamp
 
     def format(self, s):
         '''Return either t_dtype.DTYPE_DATE or t_dtype.DTYPE_TIME depending on
